@@ -136,7 +136,7 @@ let repeat =
      ^ "beginning again. When the argument is \\verb|watch|, Unison relies on "
      ^ "an external file monitoring process to synchronize whenever a change "
      ^ "happens.  You can combine the two with a \\verb|+| character to use "
-     ^ "file monitoring and also do a full scan every specificed number of "
+     ^ "file monitoring and also do a full scan every specified number of "
      ^ "seconds.  For example, \\verb|watch+3600| will react to changes "
      ^ "immediately and additionally do a full scan every hour.")
     (fun _ -> parseRepeat)
@@ -226,7 +226,7 @@ let replicaContent2string rc sep =
       "absent"
   | _, `Unchanged ->
       "unchanged "
-     ^(Util.truncateString (Fileinfo.type2string rc.typ) 7)
+     ^(Util.padto 7 (Util.truncateString (Fileinfo.type2string rc.typ) 7))
      ^ sep
      ^ replicaContentDesc rc
   | `ABSENT, `Deleted -> "deleted"
@@ -353,10 +353,10 @@ let reconItem2string oldPath theRI status =
 let exn2string e =
   match e with
      Sys.Break      -> "Terminated!"
-   | Util.Fatal(s)  -> Printf.sprintf "Fatal error: %s" s
-   | Util.Transient(s) -> Printf.sprintf "Error: %s" s
+   | Util.Fatal s   -> s
+   | Util.Transient s -> s
    | Unix.Unix_error (err, fun_name, arg) ->
-       Printf.sprintf "Uncaught unix error: %s failed%s: %s%s\n%s"
+       Printf.sprintf "Uncaught unix error (please report a bug): %s failed%s: %s%s\n%s"
          fun_name
          (if String.length arg > 0 then Format.sprintf " on \"%s\"" arg else "")
          (Unix.error_message err)
@@ -364,9 +364,14 @@ let exn2string e =
             Unix.EUNKNOWNERR n -> Format.sprintf " (code %d)" n
           | _                  -> "")
          (Printexc.get_backtrace ())
+   | Stack_overflow ->
+       "Stack overflow. This could indicate a programming error.\n\n\
+         Technical information in case you need to report a bug:\n"
+       ^ (Printexc.get_backtrace ())
    | Invalid_argument s ->
-       Printf.sprintf "Invalid argument: %s\n%s" s (Printexc.get_backtrace ())
-   | other -> Printf.sprintf "Uncaught exception %s\n%s"
+       Printf.sprintf "Invalid argument (please report a bug): %s\n%s"
+         s (Printexc.get_backtrace ())
+   | other -> Printf.sprintf "Uncaught exception (please report a bug): %s\n%s"
        (Printexc.to_string other) (Printexc.get_backtrace ())
 
 (* precondition: uc = File (Updates(_, ..) on both sides *)
@@ -649,12 +654,15 @@ let transportStart () = Transport.logStart ()
 let transportFinish () = Transport.logFinish ()
 
 let transportItems items pRiThisRound makeAction =
+  if Abort.isAll () then () else
   let waiter = Lwt.wait () in
   let outstanding = ref 0 in
   let starting () = incr outstanding in
   let completed () =
     decr outstanding;
-    if !outstanding = 0 then begin
+    if !outstanding = 0 || (!outstanding = 1 && Abort.isAll ()) then begin
+      (* If there is a stop request then we might not get the [completed]
+         notice for the dispense loop itself, so we also stop at count 1. *)
       try Lwt.wakeup waiter () with Invalid_argument _ -> ()
     end
   in
@@ -682,7 +690,10 @@ let transportItems items pRiThisRound makeAction =
   in
   let rec dispenseAction () =
     let i = !idx in
-    if i < im then begin
+    if Abort.isAll () then begin
+      stopDispense ();
+      dispenseDone ()
+    end else if i < im then begin
       let item = items.(i) in
       incr idx;
       if pRiThisRound item then
